'''
Structure definitions for the OSX MachO binary format.
'''

import vstruct
import vstruct.primitives as vs_prim

from vstruct.defs.macho.const import *

from vstruct.defs.macho.fat import *
from vstruct.defs.macho.loader import *

